<?php
/*
 * Class: EdlLinkFaviconCache
 * @author: Edward de Leau <e@leau.net>
 * @copyright: Edward de Leau, http://edward.de.leau.net
 * 
 *  Caching of favicons based on a domain e.g. \com\delicious\www\[cache location]
 *  so always a url is passed to determine the cache location on
 */

if (!class_exists("EdlLinkFaviconCache")) {
	class EdlLinkFaviconCache {
	
		public $faviconcachedir;
		public $url;
		public $physical_location;
		public $relative_location;
		public $file_name;
		public $file_content;
		public $physical_location_and_file;
		public $faviconcacheurl;
		
		/*
		 * constructor
		 */
		public function __construct($url, $fileName, $faviconcachedir, $faviconcacheurl) 
		{ 		

			$this->faviconcacheurl = $faviconcacheurl;		
			$this->faviconcachedir =  $faviconcachedir;
						
			$this->SetUrl($url);
			if ($fileName)
			{			
				$this->SetFileName($fileName);
			}		
			$this->physical_location_and_file = $this->physical_location . $this->file_name;
		}
		
		/*
		 * based on the url we can find the directory we should write to
		 */
		function SetUrl($url) {
			$write_loc = '';
			if ($url) 
			{
				$url_array = Array();		
				$this->url = $url;			
				// and now we also know the cache location			
				// 1. first make the path for the root domain:
				$url_array = parse_url($url);
				$segments = array_reverse(explode('.',$url_array['host']));
				
				foreach($segments as $segment) {
					$write_loc .= '/' . $segment;
				}				
				// 2. then check for special paths that we want to use for specific
				//    path based root domains we specify e.g. apps.facebook.com			
				if ($url_array['host'] === 'apps.facebook.com') {
					$write_loc .= $url_array['path'];
				}
			} 					
			// @todo strip first slash from $write_loc
			$write_loc = substr($write_loc, 1);
			$this->relative_location = $this->faviconcacheurl . $write_loc . '/';			
			$this->physical_location = $this->faviconcachedir . $write_loc;
			$this->physical_location_and_file = $this->physical_location . '/'.  $this->file_name;
						
			return true;
		}
		
		/*
		 * the filename of the file we want to cache
		 */
		function SetFileName($filename) 
		{
			$this->file_name = $filename;
			$this->physical_location_and_file = $this->physical_location . '/' . $this->file_name;			
			return true;
		}
		
		/*
		 *
		 */
		function SetContent($filecontent) 
		{
			$this->file_content = $filecontent;
			return true;
		}
		
		/*
		 *
		 */
		function GetRelativeLocation() {
			return $this->relative_location;
		}
		
		/*
		 *
		 */		
		function GetPhysicalPathAndFile() {
			return $this->physical_location_and_file;
		}
		
		/*
		 * create cache Directory
		 */
		function CreateCacheDirectory() 
		{		
			$rs = @mkdir($this->physical_location, 0755, true);
			return $rs;
		}
		
		/*
		 * check if the file exists we want to cache
		 * if it does not write it
		 * always return true unless error
		 */
		function FileExistInCache() 
		{		
			if (file_exists($this->physical_location_and_file))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		/*
		 * write the file to cache
		 */
		function WriteFileToCache() 
		{
			// for safety create the check / create the dir
			$this->CreateCacheDirectory();
				
			//echo 	$this->physical_location_and_file; exit;
				
			$fh = fopen($this->physical_location_and_file, 'w') 
				or die("can't open file " . $this->physical_location_and_file);
			fwrite($fh, $this->file_content);
			fclose($fh);		
		}
		
		/*
		 * read cache
		 */
		function ReadFileFromCache() 
		{
			$data = @file_get_contents($this->physical_location_and_file);
			return $data;
		}
			
		/*
		 * we dont want to serialize everything since we also want to use the "cache" for directly
		 * calling images and do some datamining
		 */
		function CheckCache($callback_function, $use_cache=true, $serialize=true)
		{
			if ($use_cache && $this->FileExistInCache())
			{
				if ($serialize) 
				{
					$obj = unserialize($this->ReadFileFromCache());
				}
				else
				{
					$obj = $this->ReadFileFromCache();
				}
			} 
			else
			{
				if (is_callable($callback_function)) 
				{
					$obj = call_user_func($callback_function);	
				}
				else
				{
					echo "error: callback function not callable";
				}
				if ($serialize) 
				{				
					$this->SetContent(serialize($obj));
				}
				else
				{
					$this->SetContent($obj);
				}
				$this->WriteFileToCache();
			}
			
			return $obj;
		}
		
		/*
		 * when we already have the data we only want to update the 
		 * cache (probably) so check if there is data at all and then
		 * just update the cache
		 * 
		 * note problem resolution: in some cases there could be multiple files stored e.g.
		 * a.bmp and a.gif while only one is true.
		 */		
		function CheckCacheData($data, $use_cache=true, $serialize=true)
		{
			if ($use_cache && $this->FileExistInCache())
			{
				if ($serialize) 
				{
					$obj = unserialize($this->ReadFileFromCache());
				}
				else
				{
					$obj = $this->ReadFileFromCache();
				}
			} 
			else
			{
				$obj = $data;
				if ($data) 
				{
					if ($serialize) 
					{				
						$this->SetContent(serialize($obj));
					}
					else
					{
						$this->SetContent($obj);
					}
					$this->WriteFileToCache();
				}	
			}
			
			return $obj;
		}
		
		
	}
}