<?php
/*
 * Class: EdlLinkFavicon
 * @author: Edward de Leau <e@leau.net>
 * @copyright: Edward de Leau, http://edward.de.leau.net
 */

if (!class_exists("EdlLinkFaviconContent") && class_exists("EdlHtmlHref")) 
{
	class EdlLinkFaviconContent extends EdlHtmlHref
	{
		
		const IMAGE ='favicon.png';
		const GOOGLE_ICON_URL = 'http://www.google.com/s2/favicons?domain=';
		private $url;
		private $md5_url;
		private $local_image_url;
		private $cache_file_name;
		public $mStrReplaceType;
		
		private $mArrLinkFaviconOptions = array();
		
		// set DEBUG to 1 for now to debug
		// :TODO add more debug levels for more kinds of tests
		public $mDebugLevel = 0;
		
		/*
  		 * constructor
		 */
		public function __construct($ReplaceContext) 		
		{
			// some day this will be an ENUM
		
			if ($ReplaceContext=='widgets')
			{
				$this->mStrReplaceType = 'widgets';
			}
			else
			{
				$this->mStrReplaceType = 'content';
			}
		}
		function EdlLinkFaviconContent()
		{
			return $this->__construct();
		}
		
		public function SetDebugLevel($boolDebugLevel) 
		{
			$this->mDebugLevel = $boolDebugLevel;	
			if ($this->mDebugLevel != 1):			
				echo "*** debug level is now off<br />";
			else:
				echo "*** debug level is now on<br />";
			endif;
		}
		
		/*
		 * function: Init
		 * gets the options set in the admin panel
		 *
		 */
		public function Init($mArrLinkFaviconOptions = array()) 
		{
			if ($this->mDebugLevel==1):
				$this->mArrLinkFaviconOptions['not_used_yet'] = 1;				
			else:		
				$this->mArrLinkFaviconOptions = $mArrLinkFaviconOptions;
			endif;
			
			$this->cache_file_name = self::IMAGE;
			
			parent::SetNotAllowedExtension($this->mArrLinkFaviconOptions['wp_favicons_exclude_filetypes']);
		}
			
		/**
		 * function: Add Filter
		 *
		 */
		public function AddFilter() 
		{
			// replace in content
			if ($this->mStrReplaceType == 'content') {
				if ($this->mArrLinkFaviconOptions['wp_favicons_show_in_content'])			
				{
					add_filter('the_content', array($this,'ReplaceAll'), 9);
				}				
			}
			else
			// replace in widgets
			{
				if ($this->mArrLinkFaviconOptions['wp_favicons_show_in_widgets'])			
				{
					// normal text in widgets:
					add_filter('widget_text', array($this,'ReplaceAll'), 9);
					// link list:
					add_filter('wp_list_bookmarks',array($this,'ReplaceAll'), 9);
					// undoubtly more rfcs for specific widgets
				}	
			}				
		}
						
		/*
		 * callback function for the cache
		 */
		function HttpDownloadFeed()
		{
			$parsed_url = parse_url($this->url); 	
			$data_r = wp_remote_get(self::GOOGLE_ICON_URL . $parsed_url['host']);
			$data = $data_r['body'];
			return $data;
		}
		
		/**
		 * function: GetIconHTML
		 * creates and returns the Icon HTML to be added after the href
		 */
		function GetIconHTML()
		{
			$str_favicon_string = '<img src="';
		
			// check if a cache should be used
			if ($this->mArrLinkFaviconOptions['wp_favicons_use_cache']) 
			{
				$favicon_cache = new EdlLinkFaviconCache(
					$this->url,
					'favicon.png',
					$this->mArrLinkFaviconOptions['wp_favicons_upload_dir_cache'],
					$this->mArrLinkFaviconOptions['wp_favicons_upload_url_cache']);
				
				// todo add more priorities in image namings to allow more favicon additions
				$icon_type = array( 'image/bmp'    => 'favicon.bmp',
									'image/png'    => 'favicon.png',
									'image/gif'    => 'favicon.gif',
									'image/x-icon' => 'favicon.ico',
									'image/jpeg'   => 'favicon.jpg');

				$image_found = 0;
				foreach($icon_type as $key => $value)
				{
					$favicon_cache->SetFileName($value);
					if ($favicon_cache->FileExistInCache())
					{
						++$image_found;
						$this->local_image_url = $favicon_cache->GetRelativeLocation() . $value;					
					}
				}				
				if ($image_found==0)
				{
					$favicon_cache->SetFileName($this->cache_file_name);
					$obj = $favicon_cache->CheckCache(array($this,'HttpDownloadFeed'),$this->use_cache,false);
					$this->local_image_url = $favicon_cache->GetRelativeLocation() . $this->cache_file_name;
				}
				
				$str_favicon_string .= $this->local_image_url;
			}
			else 
			{
				$parsed_url = parse_url($this->url);
				$str_favicon_string .= self::GOOGLE_ICON_URL . $parsed_url[host];
			}
		
			$str_favicon_string .= '" alt="wp-favicon" class="';
			
			if ($this->mStrReplaceType == 'content') 
			{
				$str_favicon_string .= 'wp-favicon-content';
			}	
			else
			{
				$str_favicon_string .= 'wp-favicon-widgets';
			}						
			$str_favicon_string .= '" />';
			return $str_favicon_string;
		}
		
		/*
		 *
		 */
		function CreateAndGetIcon() 
		{
			// Add the icon html code and the text html code			
			return $this->GetIconHtml();
		}
		
		/*
		 *
		 */
		public function SpecificHrefBefore($strHtmlUri) 
		{
			$this->url = $strHtmlUri;
			$this->md5_url = md5($this->url);
			return $this->CreateAndGetIcon();		
		}
				
	} 
}